@@ -2,7 +2,8 @@
 	Integrate.c
 		integrate over the unit hypercube
 		this file is part of Cuhre
-		last modified 28 Mar 13 th
+		checkpointing by B. Chokoufe
+		last modified 2 May 13 th
 */
 
 
@@ -16,23 +17,32 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     Region region[POOLSIZE];
   } Pool;
 
-  count dim, comp, ncur, ipool, npool;
-  int fail;
-  Totals totals[NCOMP];
+  count dim, comp, ipool, npool;
   Pool *cur = NULL, *pool;
+  ccount poolsize = sizeof(Pool);
   Region *region;
+  int fail;
+  struct {
+    signature_t signature;
+    count nregions, ncur;
+    number neval;
+    Totals totals[NCOMP];
+  } state;
+  StateDecl;
 
   if( VERBOSE > 1 ) {
-    char s[256];
+    char s[512];
     sprintf(s, "Cuhre input parameters:\n"
       "  ndim " COUNT "\n  ncomp " COUNT "\n"
       "  epsrel " REAL "\n  epsabs " REAL "\n"
       "  flags %d\n  mineval " NUMBER "\n  maxeval " NUMBER "\n"
-      "  key " COUNT,
+      "  key " COUNT "\n"
+      "  statefile \"%s\"",
       t->ndim, t->ncomp,
       t->epsrel, t->epsabs,
       t->flags, t->mineval, t->maxeval,
-      t->key);
+      t->key,
+      t->statefile);
     Print(s);
   }
 
@@ -48,31 +58,55 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
   if( (fail = setjmp(t->abort)) ) goto abort;
 
-  Alloc(cur, 1);
-  cur->next = NULL;
-  ncur = 1;
+  StateSetup(t);
 
-  region = cur->region;
-  region->div = 0;
-  for( dim = 0; dim < t->ndim; ++dim ) {
-    Bounds *b = &region->bounds[dim];
-    b->lower = 0;
-    b->upper = 1;
+  if( StateReadTest(t) ) {
+    StateReadOpen(t, fd) {
+      Pool *prev = NULL;
+      int size;
+      if( read(fd, &state, sizeof state) != sizeof state ||
+        state.signature != StateSignature(t, 4) ) break;
+      t->neval = state.neval;
+      t->nregions = state.nregions;
+      do {
+        Alloc(cur, 1);
+        cur->next = prev;
+        prev = cur;
+        size = read(fd, cur, poolsize);
+      } while( size == poolsize );
+      if( size != (char *)&cur->region[state.ncur] - (char *)cur ) break;
+    } StateReadClose(t, fd);
   }
 
-  Sample(t, region);
+  if( ini ) {
+    Alloc(cur, 1);
+    cur->next = NULL;
+    state.ncur = t->nregions = 1;
 
-  for( comp = 0; comp < t->ncomp; ++comp ) {
-    Totals *tot = &totals[comp];
-    Result *r = &region->result[comp];
-    tot->avg = tot->lastavg = tot->guess = r->avg;
-    tot->err = tot->lasterr = r->err;
-    tot->weightsum = 1/Max(Sq(r->err), NOTZERO);
-    tot->avgsum = tot->weightsum*r->avg;
-    tot->chisq = tot->chisqsum = tot->chisum = 0;
+    region = cur->region;
+    region->div = 0;
+    for( dim = 0; dim < t->ndim; ++dim ) {
+      Bounds *b = &region->bounds[dim];
+      b->lower = 0;
+      b->upper = 1;
+    }
+
+    t->neval = 0;
+    Sample(t, region);
+
+    for( comp = 0; comp < t->ncomp; ++comp ) {
+      Totals *tot = &state.totals[comp];
+      Result *r = &region->result[comp];
+      tot->avg = tot->lastavg = tot->guess = r->avg;
+      tot->err = tot->lasterr = r->err;
+      tot->weightsum = 1/Max(Sq(r->err), NOTZERO);
+      tot->avgsum = tot->weightsum*r->avg;
+      tot->chisq = tot->chisqsum = tot->chisum = 0;
+    }
   }
 
-  for( t->nregions = 1; ; ++t->nregions ) {
+  /* main iteration loop */
+  for( ; ; ) {
     count maxcomp, bisectdim;
     real maxratio, maxerr;
     Result result[NCOMP];
@@ -87,7 +121,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
         t->nregions, t->neval);
 
       for( comp = 0; comp < t->ncomp; ++comp ) {
-        cTotals *tot = &totals[comp];
+        cTotals *tot = &state.totals[comp];
         p += sprintf(p, "\n[" COUNT "] "
           REAL " +- " REAL "  \tchisq " REAL " (" COUNT " df)",
           comp + 1, tot->avg, tot->err, tot->chisq, t->nregions - 1);
@@ -99,7 +133,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     maxratio = -INFTY;
     maxcomp = 0;
     for( comp = 0; comp < t->ncomp; ++comp ) {
-      creal ratio = totals[comp].err/MaxErr(totals[comp].avg);
+      creal ratio = state.totals[comp].err/MaxErr(state.totals[comp].avg);
       if( ratio > maxratio ) {
         maxratio = ratio;
         maxcomp = comp;
@@ -115,7 +149,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
     maxerr = -INFTY;
     regionL = cur->region;
-    npool = ncur;
+    npool = state.ncur;
     for( pool = cur; pool; npool = POOLSIZE, pool = pool->next )
       for( ipool = 0; ipool < npool; ++ipool ) {
         Region *region = &pool->region[ipool];
@@ -126,13 +160,13 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
         }
       }
 
-    if( ncur == POOLSIZE ) {
+    if( state.ncur == POOLSIZE ) {
       Pool *prev = cur;
       Alloc(cur, 1);
       cur->next = prev;
-      ncur = 0;
+      state.ncur = 0;
     }
-    regionR = &cur->region[ncur++];
+    regionR = &cur->region[state.ncur++];
 
     regionR->div = ++regionL->div;
     FCopy(result, regionL->result);
@@ -150,7 +184,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       cResult *r = &result[comp];
       Result *rL = &regionL->result[comp];
       Result *rR = &regionR->result[comp];
-      Totals *tot = &totals[comp];
+      Totals *tot = &state.totals[comp];
       real diff, err, w, avg, sigsq;
 
       tot->lastavg += diff = rL->avg + rR->avg - r->avg;
@@ -183,10 +217,25 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
         tot->err = sqrt(sigsq);
       }
     }
+    ++t->nregions;
+
+    if( StateWriteTest(t) ) {
+      StateWriteOpen(t, fd) {
+        Pool *prev = cur;
+        state.signature = StateSignature(t, 4);
+        state.nregions = t->nregions;
+        state.neval = t->neval;
+        write(fd, &state, sizeof state);
+        while( (prev = prev->next) ) write(fd, prev, poolsize);
+fprintf(stderr, "ncur=%d  sizeof(Region)=%ld  size=%ld\n", state.ncur, 
+sizeof(Region), (char *)&cur->region[state.ncur] - (char *)cur);
+        write(fd, cur, (char *)&cur->region[state.ncur] - (char *)cur);
+      } StateWriteClose(t, fd);
+    }
   }
 
   for( comp = 0; comp < t->ncomp; ++comp ) {
-    cTotals *tot = &totals[comp];
+    cTotals *tot = &state.totals[comp];
     integral[comp] = tot->avg;
     error[comp] = tot->err;
     prob[comp] = ChiSquare(tot->chisq, t->nregions - 1);
@@ -197,7 +246,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     MLPutFunction(stdlink, "List", 2);
     MLPutFunction(stdlink, "List", t->nregions);
 
-    npool = ncur;
+    npool = state.ncur;
     for( pool = cur; pool; npool = POOLSIZE, pool = pool->next )
       for( ipool = 0; ipool < npool; ++ipool ) {
         Region const *region = &pool->region[ipool];
@@ -233,6 +282,8 @@ abort:
   FrameFree(t);
   RuleFree(t);
 
+  StateRemove(t);
+
   return fail;
 }
 
