@@ -2,7 +2,8 @@
 	Integrate.c
 		integrate over the unit hypercube
 		this file is part of Suave
-		last modified 10 Mar 13 th
+		checkpointing by B. Chokoufe
+		last modified 2 May 13 th
 */
 
 
@@ -10,24 +11,32 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 {
   TYPEDEFREGION;
 
-  count dim, comp, df;
-  int fail;
-  Result totals[NCOMP];
+  count dim, comp;
   Region *anchor = NULL, *region = NULL;
+  int fail;
+  struct {
+    signature_t signature;
+    count nregions, df;
+    number neval;
+    Result totals[NCOMP];
+  } state;
+  StateDecl;
 
   if( VERBOSE > 1 ) {
-    char s[256];
+    char s[512];
     sprintf(s, "Suave input parameters:\n"
       "  ndim " COUNT "\n  ncomp " COUNT "\n"
       "  epsrel " REAL "\n  epsabs " REAL "\n"
       "  flags %d\n  seed %d\n"
       "  mineval " NUMBER "\n  maxeval " NUMBER "\n"
-      "  nnew " NUMBER "\n  flatness " REAL,
+      "  nnew " NUMBER "\n  flatness " REAL "\n"
+      "  statefile \"%s\"\n",
       t->ndim, t->ncomp,
       t->epsrel, t->epsabs,
       t->flags, t->seed,
       t->mineval, t->maxeval,
-      t->nnew, t->flatness);
+      t->nnew, t->flatness,
+      t->statefile);
     Print(s);
   }
 
@@ -42,32 +51,66 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   t->epsabs = Max(t->epsabs, NOTZERO);
   IniRandom(t);
 
-  RegionAlloc(t, anchor, t->nnew, t->nnew);
-  anchor->next = NULL;
-  anchor->div = 0;
+  StateSetup(t);
 
-  for( dim = 0; dim < t->ndim; ++dim ) {
-    Bounds *b = &anchor->bounds[dim];
-    b->lower = 0;
-    b->upper = 1;
-    b->mid = .5;
-
-    if( dim == 0 ) {
-      count bin;
-      /* define the initial distribution of bins */
-      for( bin = 0; bin < NBINS; ++bin )
-        b->grid[bin] = (bin + 1)/(real)NBINS;
-    }
-    else Copy(b->grid, anchor->bounds[0].grid, NBINS);
+  if( StateReadTest(t) ) {
+    StateReadOpen(t, fd) {
+      count iregion;
+      size_t *regsize, totsize;
+      Region **last = &anchor;
+      if( read(fd, &state, sizeof state) != sizeof state ||
+        state.signature != StateSignature(t, 2) ) break;
+      t->nregions = state.nregions;
+      totsize = t->nregions*sizeof *regsize;
+      regsize = alloca(totsize);
+      if( read(fd, regsize, totsize) != totsize ) break;
+      for( iregion = 0; iregion < t->nregions; ++iregion )
+        totsize += regsize[iregion];
+      if( st.st_size != sizeof state + totsize ) break;
+      for( iregion = 0; iregion < t->nregions; ++iregion ) {
+        Region *reg;
+        MemAlloc(reg, regsize[iregion]);
+        read(fd, reg, regsize[iregion]);
+        *last = reg;
+        last = &reg->next;
+      }
+      *last = NULL;
+    } StateReadClose(t, fd);
+    t->neval = state.neval;
+    t->rng.skiprandom(t, t->neval);
   }
 
-  Sample(t, t->nnew, anchor, anchor->w,
-    anchor->w + t->nnew,
-    anchor->w + t->nnew + t->ndim*t->nnew);
-  df = anchor->df;
-  FCopy(totals, anchor->result);
+  if( ini ) {
+    RegionAlloc(t, anchor, t->nnew, t->nnew);
+    anchor->next = NULL;
+    anchor->div = 0;
+    t->nregions = 1;
 
-  for( t->nregions = 1; ; ++t->nregions ) {
+    for( dim = 0; dim < t->ndim; ++dim ) {
+      Bounds *b = &anchor->bounds[dim];
+      b->lower = 0;
+      b->upper = 1;
+      b->mid = .5;
+
+      if( dim == 0 ) {
+        count bin;
+        /* define the initial distribution of bins */
+        for( bin = 0; bin < NBINS; ++bin )
+          b->grid[bin] = (bin + 1)/(real)NBINS;
+      }
+      else Copy(b->grid, anchor->bounds[0].grid, NBINS);
+    }
+
+    t->neval = 0;
+    Sample(t, t->nnew, anchor, anchor->w,
+      anchor->w + t->nnew,
+      anchor->w + t->nnew + t->ndim*t->nnew);
+    state.df = anchor->df;
+    FCopy(state.totals, anchor->result);
+  }
+
+  /* main iteration loop */
+  for( ; ; ) {
     Var var[NDIM][2], *vLR;
     real maxratio, maxerr, minfluct, bias, mid;
     Region *regionL, *regionR, *reg, **parent, **par;
@@ -84,10 +127,10 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
         t->nregions, t->neval);
 
       for( comp = 0; comp < t->ncomp; ++comp ) {
-        cResult *tot = &totals[comp];
+        cResult *tot = &state.totals[comp];
         p += sprintf(p, "\n[" COUNT "] " 
           REAL " +- " REAL "  \tchisq " REAL " (" COUNT " df)",
-          comp + 1, tot->avg, tot->err, tot->chisq, df);
+          comp + 1, tot->avg, tot->err, tot->chisq, state.df);
       }
 
       Print(s);
@@ -96,7 +139,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     maxratio = -INFTY;
     maxcomp = 0;
     for( comp = 0; comp < t->ncomp; ++comp ) {
-      creal ratio = totals[comp].err/MaxErr(totals[comp].avg);
+      creal ratio = state.totals[comp].err/MaxErr(state.totals[comp].avg);
       if( ratio > maxratio ) {
         maxratio = ratio;
         maxcomp = comp;
@@ -122,6 +165,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       }
     }
 
+    /* find the bisectdim which minimizes the fluctuations */
     Fluct(t, var[0],
       region->bounds, region->w, region->n, maxcomp,
       region->result[maxcomp].avg, Max(maxerr, t->epsabs));
@@ -140,6 +184,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       }
     }
 
+    /* apply stratified sampling to distribute points in bisected region */
     vLR = var[bisectdim];
     minfluct = vLR[0].fluct + vLR[1].fluct;
     nnewL = IMax(
@@ -187,7 +232,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       if( n && final ) wlast = w, flast = f;
     }
 
-    Reweight(t, region->bounds, wlast, flast, f, totals);
+    Reweight(t, region->bounds, wlast, flast, f, state.totals);
     XCopy(regionL->bounds, region->bounds);
     XCopy(regionR->bounds, region->bounds);
 
@@ -201,13 +246,13 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     Sample(t, nnewL, regionL, wL, xL, fL);
     Sample(t, nnewR, regionR, wR, xR, fR);
 
-    df += regionL->df + regionR->df - region->df;
+    state.df += regionL->df + regionR->df - region->df;
 
     for( comp = 0; comp < t->ncomp; ++comp ) {
       cResult *r = &region->result[comp];
       Result *rL = &regionL->result[comp];
       Result *rR = &regionR->result[comp];
-      Result *tot = &totals[comp];
+      Result *tot = &state.totals[comp];
       real diff, sigsq;
 
       tot->avg += diff = rL->avg + rR->avg - r->avg;
@@ -230,13 +275,30 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
     free(region);
     region = NULL;
+    ++t->nregions;
+
+    if( StateWriteTest(t) ) {
+      StateWriteOpen(t, fd) {
+        Region *reg;
+        size_t regsize[t->nregions], *s = regsize;
+        state.signature = StateSignature(t, 2);
+        state.neval = t->neval;
+        state.nregions = t->nregions;
+        for( reg = anchor; reg; reg = reg->next )
+          *s++ = reg->size;
+        write(fd, &state, sizeof state);
+        write(fd, regsize, sizeof regsize);
+        for( reg = anchor; reg; reg = reg->next )
+          write(fd, reg, reg->size);
+      } StateWriteClose(t, fd);
+    }
   }
 
   for( comp = 0; comp < t->ncomp; ++comp ) {
-    cResult *tot = &totals[comp];
+    cResult *tot = &state.totals[comp];
     integral[comp] = tot->avg;
     error[comp] = tot->err;
-    prob[comp] = ChiSquare(tot->chisq, df);
+    prob[comp] = ChiSquare(tot->chisq, state.df);
   }
 
 #ifdef MLVERSION
@@ -277,6 +339,8 @@ abort:
   WaitCores(t);
   ShmFree(t);
 
+  StateRemove(t);
+
   return fail;
 }
 
